/*! \file csmaMac.c
\brief Carrier-sensing random access MAC.

@version 3.0 (SVN rev 724)
@author Chris Hunter

The csmaMac is a modified ALOHA MAC that
serves as an example for novel MAC
development. Nodes transmit whenever
they have information to transmit, and only
move on to the next packet once the original
transmit is acknowledged (ACKed). If no ACK
is received, a collision is inferred and the
packet is re-transmitted.

By default, the MAC also implements carrier-
sensing multiple-access with collision-
avoidance (CSMA-CA). This functionality is
built into hardware peripherals in the project
so very little software state is affected.

In its current state, the project acts as
a true ethernet MAC-level wireless bridge.
Any ethernet activity that appears on one
WARP will be sent to another via the custom
wireless link.

*/

#include "warpmac.h"
#include "warpphy.h"
#include "xparameters.h"
#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include "xutil.h"
#include "xcache_l.h"
#include "xexception_l.h"
#include "xcache_l.h"
#include "xutil.h"
#include "xexception_l.h"


///Routing table with agreed upon mapping between dipswitches and physical addresses
typedef struct {
	unsigned char addr[6];
} route;

///Byte array containing physical MAC address of this node
unsigned char myAddr[6];
///Instance of the routing table
route routeTable[16];
///Index to the routing table that identifies this node
unsigned char myID;

///Buffer for holding a packet-to-xmit across multiple retransmissions
Macframe txBuffer; 
///Buffer to hold received packet
Macframe rxBuffer;

///Data packet with payload meant for Ethernet transmission
#define DATAPACKET 1
///Acknowedgement packet meant for halting retransmissions
#define ACKPACKET 0


///@brief Callback for the expiration of timers
///
///This function is responsible for handling #TIMEOUT and #BACKOFF.
///The job responsibilities of this function are to:
///-increase the contention window upon the expiration of a #TIMEOUT
///-initiate a #BACKOFF timer upon the expiration of a #TIMEOUT
///-retransmit a packet upon the expiration of a #BACKOFF
///@param timerType #TIMEOUT or #BACKOFF
int timerExpire(unsigned char timerType){
	int status;
	switch(timerType){
		case TIMEOUT:
			if(txBuffer.isNew){
				status = warpmac_incrementResend(&txBuffer);
				if(status == 0){
					warpmac_enableEthernetInterrupt();
					return 0;
				}
				warpmac_setTimer(BACKOFF);
				return 0;
			}
			break;
		case BACKOFF:
			if(warpmac_carrierSense()){
				warpmac_sendOfdm(&txBuffer);
				warpmac_setTimer(TIMEOUT);
			}
			else{
				warpmac_setTimer(BACKOFF);
			}
			break;
			return 0;
	}
}


///@brief Callback for the reception of Ethernet packets
///
///This function is called by the ethernet MAC drivers
///when a packet is available to send. This function fills
///the Macframe transmit buffer with the packet and sends
///it over the OFDM link
///@param length Length, in bytes, of received Ethernet frame
///@param payload address of first byte in Ethernet payload.
int ethernet_callback(Xuint32 length, char* payload){
	warpmac_disableEthernetInterrupt();
	txBuffer.header.currReSend = 0;
	txBuffer.isNew = 1;
	txBuffer.header.length = length;
	txBuffer.header.pktType = DATAPACKET;

	//Set the modulation scheme for the packet's full-rate symbols
	txBuffer.header.fullRate = QPSK;

	//Copy in the packet's destination MAC address
	//Hard-coded as this node's partner node
	memcpy(txBuffer.header.destAddr,routeTable[(myID+1)%2].addr,6);
	
	
	if(warpmac_carrierSense()){
		warpmac_sendOfdm(&txBuffer);
		warpmac_setTimer(TIMEOUT);
	}
	else{
		warpmac_setTimer(BACKOFF);
	}
	return 0;
}

///@brief Callback for the reception of bad wireless packets
///
///@param packet Pointer to received Macframe
int receiveBadPacket(Macframe* packet) {
	warpmac_incrementLEDLow();
}

///@brief Callback for the reception of good wireless packets
///
///This function first determines if the destination of the packet was this node.
///If so, it sends an #ACKPACKET if the received packet was a #DATAPACKET,
///and pushes the received packet over Ethernet. If the received packet
///was an #ACKPACKET, the #TIMEOUT is stopped. 
///@param packet Pointer to received Macframe
int receiveGoodPacket(Macframe* packet) {
	
	warpmac_incrementLEDHigh();

	if(warpmac_addressedToMe(packet)){
		Macframe ackPacket;
		switch(packet->header.pktType){
			case ACKPACKET:
						
				if(warpmac_inTimeout()){
					warpmac_clearTimer(TIMEOUT);
					txBuffer.header.currReSend = 0;
					txBuffer.isNew = 0;
					warpmac_enableEthernetInterrupt();
				}

				break;
			case DATAPACKET:
				warpmac_leftHex(packet->header.currReSend);
				ackPacket.header.length = 0;
				ackPacket.header.pktType = ACKPACKET;
				ackPacket.header.fullRate = QPSK;
				memcpy(ackPacket.header.srcAddr,myAddr,6);
				memcpy(ackPacket.header.destAddr,packet->header.srcAddr,6);
				warpmac_setTxBuffer(2);
				warpmac_sendOfdm(&ackPacket);
				warpmac_setTxBuffer(1);
				
				packet->header.currReSend = 0;
				packet->isNew = 1;
				warpmac_phyInterruptClear();
				warpmac_sendEthernet(packet);
				packet->header.currReSend = 0;
				packet->isNew = 0;
				
				break;
		}
	}
return 0;
}

///@brief Main function
///
///This function configures MAC parameters, enables the underlying frameworks, and then loops forever.
int main(){
	unsigned char antSel = 0;

	//Read Dip Switch value from FPGA board.
	//This value will be used as an index into the routing table for other nodes
	myID = warpmac_getMyId();

	//Create an arbitrary address for this node
	unsigned char tmpAddr[6] = {0x16,0x24,0x63,0x53,0xe2,0xc2+myID};

	memcpy(myAddr,tmpAddr,6);

	//Fill an arbitrary routing table so that nodes know each others' addresses
	unsigned char i;
	for(i=0;i<16;i++){
		routeTable[i].addr[0] = myAddr[0];
		routeTable[i].addr[1] = myAddr[1];
		routeTable[i].addr[2] = myAddr[2];
		routeTable[i].addr[3] = myAddr[3];
		routeTable[i].addr[4] = myAddr[4];
		routeTable[i].addr[5] = myAddr[5]+i-myID;
	}

	//Initialize the framework
	warpmac_init();

	warpmac_setMacAddr(&myAddr);
	
	warpmac_setMaxResend(8);
	warpmac_setMaxCW(5);
	warpmac_setTimeout(160);
	warpmac_setSlotTime(9);
	
	warpmac_setRxBuffer(&rxBuffer,0);
	warpmac_setTxBuffer(1);
	
	memcpy(txBuffer.header.srcAddr,myAddr,6);

	warpmac_setGoodPacketHandler(receiveGoodPacket);
	warpmac_setBadPacketHandler(receiveBadPacket);
	warpmac_setTimerHandler(timerExpire);
	warpmac_setEmacHandler(ethernet_callback);

	warpmac_setChannel(GHZ_2,8);

	warpmac_enableCSMA();
	warpmac_enableEthernetInterrupt();

	//Set the modulation scheme use for base rate (header) symbols
	warpmac_setBaseRate(QPSK);


	while(1){
	}

	return;
}




