/*! \file warpmac.h
\brief Header file for the WARPMAC Framework.

@version 3.0 (SVN rev 674)
@author Chris Hunter

This header file contains the macros, function prototypes, and typedefs required for WARPMAC.
*/



/*! \mainpage WARP APIs
* @version Used with Reference Design v06
*
* \section intro_sec Introduction
*
* This document is the collective API for WARPMAC (the MAC design framework), 
* WARPPHY (the PHY driver/interface), RADIO CONTROLLER, and CSMAMAC (the carrier-sensing random access 
* reference).
*
* \section start_sec Getting Started
*
* Obtain the latest reference design: http://warp.rice.edu/trac/wiki/OFDMReferenceDesign 
* In addition to being in the repository, all the source code for WARPMAC is available in 
* the reference design project and is configured to compile "as-is."
*
* \section hardcopy_sec Hardcopy
* You can download a hardcopy of this reference manual at:
* http://warp.rice.edu/WARP_API/refman.pdf
*
* \section licence_sec License
* http://warp.rice.edu/license/
*
*
*/



#ifndef WARPMAC_H
#define WARPMAC_H

#include "xstatus.h"
#include "warpphy.h"

//Define which radios get used below
//RADIOx_ADDR are defined by the radio controller driver
///Address of the first radio
#define FIRST_RADIO RADIO2_ADDR
///Address of the second radio
#define SECOND_RADIO RADIO3_ADDR

///
#define MY_XEM_MAX_FRAME_SIZE 2000

///modmask for BPSK
#define BPSK 1
///modmask for QPSK
#define QPSK 2
///modmask for 16-QAM
#define QAM_16 4
///modmask for 64-QAM
#define QAM_64 6

///5 GHz Band
#define GHZ_5 0
///2.4 GHz Band
#define GHZ_2 1

//Interrupts - Position in the interrupt vector inside the Intc peripheral
///Timer interrupt identifier
#define TMA_INTERRUPT 3
///Receive "good" interrupt identifier
#define RX_INTERRUPT 2
///Receive "bad" interrupt identifier
#define BADRX_INTERRUPT 1
///Push button interrupt identifier
#define PB_INTERRUPT 0

///Identifier for deterministic timeout countdowns
#define TIMEOUT 0
///Identifier for random backoff countdowns
#define BACKOFF 1

///Structure contains the header contents of a packet
typedef struct {
	///Physical layer header struct
	phyHeader header;
	///A flag that can be used to protect a Macframe from being overwritten.
	unsigned char isNew;
} Macframe __attribute__((__aligned__(8)));

///Structure of miscellaneous control bits needed for correct operation of the MAC.
typedef struct {
	///Timeout time of system
	volatile unsigned int timeout;
	///Smallest time interval in system
	volatile unsigned int slotTime;
	///The node's self MAC address
	volatile unsigned char selfAddr[6];
	///Maximum number of retransmissions before dropping a packet
	volatile unsigned char maxReSend;
	///Current backoff window
	volatile unsigned char currBackoff;
	///Maximum backoff window
	volatile unsigned char maxBackoff;
	///Type of timer (#TIMEOUT or #BACKOFF)
	volatile unsigned char timerType;
	///Whether the timer is currently active
	volatile unsigned char timerEnabled;
	//volatile unsigned int currBackoffTime;
	///Whether the Emac is currently enabled
	volatile unsigned char enableEthernetInt;
	///Which of the PHY's buffers is currently associated with transmission
	volatile unsigned char txBuffIndex;
	///Which of the PHY's buffers is currently associated with reception
	volatile unsigned char rxBuffIndex;
	///Constellation order of the full rate symbols (#BPSK, #QPSK, #QAM_16, or #QAM_64)
	volatile unsigned char mod_fullRateA;
	//volatile unsigned char mod_fullRateB;	
} Maccontrol;


///@brief Function to set the frequency band of operation
///
///NOTE: This is a macro wrapper around warpphy_setChannel
///@param band #GHZ_5 or #GHZ_2
///@param c Channel number
#define warpmac_setChannel(band, c) warpphy_setChannel(band,c)


///@brief Halts a currently running timer
///
///NOTE: This is a macro wrapper around ofdm_timer_stop()
#define warpmac_stopTimer() ofdm_timer_stop()

void nullHandler(void* param);
int warpmac_init();
void rxPhyGood();
void rxPhyBad();
void warpmac_phyInterruptClear();
void emacRx_int_handler(void *Callback);
void emacTx_int_handler(void *Callback);
void emacErr_int_handler(void *Callback, XStatus ErrCode);
void timer_int_handler();
int warpmac_clearTimer();
void warpmac_setTimerVal(Xuint32 clocks);
void warpmac_startTimer(unsigned char mode);
int randNum(int N);
unsigned int warpmac_carrierSense();
int warpmac_setTimer(int type);
void warpmac_sendEthernet(Macframe* packet);
void warpmac_setUpButtonHandler(void(*handler)());
void warpmac_setLeftButtonHandler(void(*handler)());
void warpmac_setRightButtonHandler(void(*handler)());
void warpmac_setMiddleButtonHandler(void(*handler)());
void warpmac_setTimerHandler(void(*handler)());
void warpmac_setEmacHandler(void(*handler)());
void warpmac_setGoodPacketHandler(void(*handler)());
void warpmac_setBadPacketHandler(void(*handler)());
void pb_int_handler();
void warpmac_incrementLEDLow();
void warpmac_incrementLEDHigh();
void warpmac_setBaseRate(unsigned char rate);
void warpmac_setFullRate(unsigned char rate);
int warpmac_sendOfdm(Macframe* packet);
int warpmac_setTxBuffer(unsigned int txBuff);
int warpmac_setRxBuffer(Macframe* packet,unsigned int rxBuff);
int warpmac_getTxBuffer(unsigned int txBuff);
int warpmac_getRxBuffer(unsigned int rxBuff);
int warpmac_incrementResend(Macframe* packet);
int warpmac_addressedToMe(Macframe* packet);
int warpmac_addressedFromThem(Macframe* packet,unsigned char* addr);
int warpmac_addressedToThem(Macframe* packet,unsigned char* addr);
void warpmac_setMaxResend(unsigned int c);
void warpmac_setMaxCW(unsigned int c);
int warpmac_setMacAddr(unsigned char* addr);
int warpmac_setTimeout(unsigned int time);
int warpmac_setSlotTime(unsigned int time);
int warpmac_inTimeout();
int warpmac_getMyId();
void warpmac_enableCSMA();
void warpmac_disableCSMA();
unsigned char sevenSegmentMap(unsigned char x);
void warpmac_leftHex(unsigned char x);
void warpmac_rightHex(unsigned char x);
void warpmac_setDebugGPIO(unsigned char val);
void warpmac_enableEthernetInterrupt();
void warpmac_disableEthernetInterrupt();




#endif


